% ==========================================================================================
% This script produces the results of "Model Averaging in Risk Management with an
% Application to Futures Markets" by M. Hashem Pesaran, Christoph Schleicher and
% Paolo Zaffaroni.
%
% Please see the documentation ("MARM documentation of Matlab code") for further details
% ==========================================================================================

if 0
    clear, clc
end

baseDirectory = 'C:\GSA\Papers\Model Averaging in Risk Management\Christoph\';
addpath(genpath([baseDirectory, 'volatility models']))
addpath([baseDirectory, 'portfolio choice'])
addpath([baseDirectory, 'active risk management'])
addpath([baseDirectory, 'passive risk management'])
addpath([baseDirectory, 'general'])
addpath([baseDirectory, 'ranks'])
addpath([baseDirectory, 'IPT tests'])

windowSize = 800;
estimationFrequency = 25;
nu = 6.7;   % some value for the DoF of the portfolio
distributions = {'t4','t5','t6','t7','t8','t10','normal'};

%% ------------------------------------------------------------------------------------------------------------
%  (1) import the raw data
%  ------------------------------------------------------------------------------------------------------------

if 0
    data = import_rawData(baseDirectory);
end

%% ------------------------------------------------------------------------------------------------------------
%  (2) generate the out-of-sample forecasts and forecast errors
%  ------------------------------------------------------------------------------------------------------------

if 0
    load([baseDirectory,'data.mat'])
    [cond_m_ar1, ret_res] = generate_AR1_forecasts(data.returns, windowSize, data);
    data.cond_m_ar1 = cond_m_ar1;   % one-step ahead mean forecast based on an AR(1)
    data.ret_res    = ret_res;      % one-step ahead forecast error base on an AR(1)
    save([baseDirectory,'data.mat'],'data')
else
    load([baseDirectory,'data.mat'])
end

%% ------------------------------------------------------------------------------------------------------------
%  (3) estimate volatility models
%  ------------------------------------------------------------------------------------------------------------

if 0
    if exist([baseDirectory,'EstimationOutput.mat'],'file')
        load([baseDirectory,'EstimationOutput.mat'])
    else
        EstimationOutput = struct;
    end

    [Models, modelNames] = listOfModels;
    failedModels = {};
    for jModel = 1:length(modelNames)
        jModelName = modelNames{jModel};
        disp(['Estimating model ', jModelName])
        EstimationOutput.(jModelName) = estimate_parameters(Models.(jModelName), data.ret_res, ...
            windowSize, estimationFrequency, jModelName);
    end
    EstimationOutput = orderfields(EstimationOutput);
    save([baseDirectory,'EstimationOutput.mat'],'EstimationOutput', 'Models','modelNames')
else
    load([baseDirectory,'EstimationOutput.mat'])
end


%% ------------------------------------------------------------------------------------------------------------
%  (4) compute the in-sample likelihoods, AICs and BICs
%  ------------------------------------------------------------------------------------------------------------

if 0
    if exist([baseDirectory,'SampleLikelihoods.mat'],'file')
        load([baseDirectory,'SampleLikelihoods.mat'])
    else
        SampleLikelihoods = struct; disp('starting new file SampleLikelihoods'), pause
    end
    if exist([baseDirectory,'OneStepPredictors.mat'],'file')
        load([baseDirectory,'OneStepPredictors.mat'])
    else
        OneStepPredictors = struct; disp('starting new file OneStepPredictors'), pause
    end

    for jModel = 1:length(modelNames)
        jModelName = modelNames{jModel};
        disp([num2str(jModel), ' out of ', num2str(length(modelNames)), ' (', jModelName, ')'])
        [SampleLikelihoods.(jModelName), OneStepPredictors.(jModelName)] = ...
            compute_likelihoodsAndPredictors(EstimationOutput.(jModelName), jModelName, ...
            data.ret_res, windowSize, nu, distributions);
    end
    save([baseDirectory,'SampleLikelihoods.mat'],'SampleLikelihoods')
    save([baseDirectory,'OneStepPredictors.mat'],'OneStepPredictors')
else
    load([baseDirectory,'SampleLikelihoods.mat'])
    if ~exist('OneStepPredictors','var')
        load([baseDirectory,'OneStepPredictors.mat'])
    end
end

%% ------------------------------------------------------------------------------------------------------------
%  (5) Active Risk-Management: compute the optimal strategies and evaluate their performance
%  ------------------------------------------------------------------------------------------------------------

%  ------------------------------------------------------------------------------------------------------------
%  (5.a) Single Models
%  ------------------------------------------------------------------------------------------------------------

maximumDailyLoss = 0.01;
delta = 75;
alpha = 0.01;

if 0
    if exist([baseDirectory,'ActiveRiskManagement.mat'],'file')
        load([baseDirectory,'ActiveRiskManagement.mat'])
    else
        ActiveRiskManagement = struct; disp('starting new file ActiveRiskManagement')%, pause
    end
    distributions = {'t7','normal'};
    for jModel = 1:length(modelNames)
        jModelName = modelNames{jModel};
        switch jModelName
            case 'EQMA_250'
                disp([num2str(jModel), ' out of ', num2str(length(modelNames)), ' (', jModelName, ')'])
                ActiveRiskManagement.singleModel.(jModelName) = ...
                    do_activeRiskManagementSingleModel(OneStepPredictors.(jModelName), jModelName, ...
                    data, windowSize, distributions, maximumDailyLoss, delta, alpha, EstimationOutput.(jModelName));
                figure(1)
                plot(data.dates, ActiveRiskManagement.singleModel.(jModelName).normal.cumulativeReturn, ...
                    data.dates, ActiveRiskManagement.singleModel.(jModelName).t7.cumulativeReturn)
                datetick('x',11)
                drawnow
        end
    end
    print_activeRiskManagement(ActiveRiskManagement.singleModel)
    save([baseDirectory,'ActiveRiskManagement.mat'],'ActiveRiskManagement')
else
    load([baseDirectory,'ActiveRiskManagement.mat'])
end


%% ------------------------------------------------------------------------------------------------------------
%  (5.b) Average Models
%  ------------------------------------------------------------------------------------------------------------

[Strategies, strategyNames] = listOfStrategies(Models);
if 0
    distributions = {'normal','t7'};
    for jStrategy = 1:length(strategyNames)
        jStrategyName = strategyNames{jStrategy};
        disp([num2str(jStrategy), ' out of ', num2str(length(strategyNames)), ' (', jStrategyName, ')'])
        ActiveRiskManagement.averageModel.(jStrategyName) = do_activeRiskManagementAverageModel(OneStepPredictors, ...
            SampleLikelihoods, Strategies.(jStrategyName), data, windowSize, distributions, maximumDailyLoss, ...
            delta, alpha);
        figure(1)
        plot(data.dates, ActiveRiskManagement.averageModel.(jStrategyName).normal.cumulativeReturn, ...
            data.dates, ActiveRiskManagement.averageModel.(jStrategyName).t7.cumulativeReturn)
        datetick('x',11)
        drawnow
    end
    print_activeRiskManagementAverageModel(ActiveRiskManagement.averageModel)

    save([baseDirectory,'ActiveRiskManagement.mat'],'ActiveRiskManagement')
else
    load([baseDirectory,'ActiveRiskManagement.mat'])
end

%% ------------------------------------------------------------------------------------------------------------
%  (6) Passive Risk-Management: compute the VaR statistics for a portfolio with equal weights
%  ------------------------------------------------------------------------------------------------------------

%  ------------------------------------- (6.a) Single Models --------------------------------------------------

if 0
    if exist([baseDirectory,'PassiveRiskManagement.mat'],'file')
        load([baseDirectory,'PassiveRiskManagement.mat'])
    else
        PassiveRiskManagement = struct; disp('starting new file PassiveRiskManagement')%, pause
    end
    distributions = {'t7','normal'};
    for jModel = 1:length(modelNames)
        jModelName = modelNames{jModel};
        disp([num2str(jModel), ' out of ', num2str(length(modelNames)), ' (', jModelName, ')'])
        PassiveRiskManagement.singleModel.(jModelName) = ...
            do_passiveRiskManagementSingleModel(OneStepPredictors.(jModelName), jModelName, ...
            data, windowSize, distributions, EstimationOutput.(jModelName));
        figure(1)
        plot(data.dates, PassiveRiskManagement.singleModel.(jModelName).normal.cumulativeReturn)
        datetick('x',11)
        drawnow
    end
    print_passiveRiskManagement(PassiveRiskManagement.singleModel)
    save([baseDirectory,'PassiveRiskManagement.mat'],'PassiveRiskManagement')
else
    load([baseDirectory,'PassiveRiskManagement.mat'])
end

%  ------------------------------------- (6.b) Average Models --------------------------------------------------

if 0
    distributions = {'t7','normal'};
    for jStrategy = 1:length(strategyNames)
        jStrategyName = strategyNames{jStrategy};
        disp([num2str(jStrategy), ' out of ', num2str(length(strategyNames)), ' (', jStrategyName, ')'])
        PassiveRiskManagement.averageModel.(jStrategyName) = ...
            do_passiveRiskManagementAverageModel(OneStepPredictors, SampleLikelihoods, Strategies.(jStrategyName), ...
            data, windowSize, distributions, EstimationOutput);
    end
    print_passiveRiskManagement(PassiveRiskManagement.averageModel)
    save([baseDirectory,'PassiveRiskManagement.mat'],'PassiveRiskManagement')
end

%% ------------------------------------------------------------------------------------------------------------
%  (7) Univariate Models
%  ------------------------------------------------------------------------------------------------------------

if 0
    if exist([baseDirectory,'UnivariateOutput.mat'],'file')
        load([baseDirectory,'UnivariateOutput.mat'])
    else
        UnivariateOutput = struct;
    end
    portfolioWeights = 'equal';
    [UnivariateModels, univariateModelNames] = listOfUnivariateModels;
    for jModel = 1:length(univariateModelNames)
        jModelName = univariateModelNames{jModel};
        disp(['Estimating model ', jModelName])
        UnivariateOutput.(jModelName) = estimate_univariateModels(UnivariateModels.(jModelName), data.ret_res, ...
            windowSize, estimationFrequency, jModelName, portfolioWeights, distributions);
    end
    UnivariateOutput = orderfields(UnivariateOutput);
    save([baseDirectory,'UnivariateOutput.mat'],'UnivariateOutput', 'UnivariateModels','univariateModelNames')
else
    load([baseDirectory,'UnivariateOutput.mat'])
end

if 0
    distributions = {'t7','normal'};
    for jModel = 1:length(univariateModelNames)
        jModelName = univariateModelNames{jModel};
        disp([num2str(jModel), ' out of ', num2str(length(univariateModelNames)), ' (', jModelName, ')'])
        jOneStepPredictors = UnivariateOutput.(jModelName).predictors;
        PassiveRiskManagement.univariate.singleModel.(jModelName) = ...
            do_passiveRiskManagementSingleModel(jOneStepPredictors, jModelName, ...
            data, windowSize, distributions, UnivariateOutput.(jModelName));
    end
    print_passiveRiskManagement(PassiveRiskManagement.univariate.singleModel)
    save([baseDirectory,'PassiveRiskManagement.mat'],'PassiveRiskManagement')
end

if 0
    [UnivariateStrategies, univariateStrategyNames] = listOfStrategies(UnivariateModels);
    distributions = {'t7','normal'};
    for jStrategy = 1:length(univariateStrategyNames)
        jStrategyName = univariateStrategyNames{jStrategy};
        disp([num2str(jStrategy), ' out of ', num2str(length(univariateStrategyNames)), ' (', jStrategyName, ')'])
        PassiveRiskManagement.univariate.averageModel.(jStrategyName) = ...
            do_passiveRiskManagementAverageModel_univariate(UnivariateOutput, UnivariateStrategies.(jStrategyName), ...
            data, windowSize, distributions);
    end
    print_passiveRiskManagement(PassiveRiskManagement.univariate.averageModel)
    save([baseDirectory,'PassiveRiskManagement.mat'],'PassiveRiskManagement')
end


%% ------------------------------------------------------------------------------------------------------------
%  (8) Compute the ranks and average Bayesian weights
%  ------------------------------------------------------------------------------------------------------------

if 0
    if exist([baseDirectory,'Ranks.mat'],'file')
        load([baseDirectory,'Ranks.mat'])
    else
        Ranks = struct;
        BayesianWeights = struct;
    end
    [Ranks.multivariate, BayesianWeights.multivariate] = ...
        get_ranks(SampleLikelihoods,modelNames,distributions,windowSize,data);
    save([baseDirectory,'Ranks'],'Ranks','BayesianWeights')
else
    load([baseDirectory,'Ranks'])
end

if 0
    [Ranks.univariate, BayesianWeights.univariate] = ...
        get_ranks_univariate(UnivariateOutput,distributions,windowSize,data);
    save([baseDirectory,'Ranks'],'Ranks','BayesianWeights')
end

%% ------------------------------------------------------------------------------------------------------------
%  (9) Compute the statistical diagnostic tests based on the integral probability transform
%  ------------------------------------------------------------------------------------------------------------

%  ------------------------ multivariate - single model -----------------------------------------------------
if 0
    if exist([baseDirectory,'IPT_tests.mat'],'file')
        load([baseDirectory,'IPT_tests.mat'])
    else
        IPT_tests = struct; disp('starting new file IPT_tests')
    end
    distributions = {'t7','normal'};
    for jModel = 1:length(modelNames)
        jModelName = modelNames{jModel};
        disp([num2str(jModel), ' out of ', num2str(length(modelNames)), ' (', jModelName, ')'])
        IPT_tests.multivariate.singleModel.(jModelName) = ...
            do_IPT_tests(OneStepPredictors.(jModelName), jModelName, ...
            data, windowSize, distributions, EstimationOutput.(jModelName));
    end
    save([baseDirectory,'IPT_tests.mat'],'IPT_tests')
else
    load([baseDirectory,'IPT_tests.mat'])
end

%  ------------------------ univariate - single model -----------------------------------------------------
if 0
    distributions = {'t7','normal'};
    for jModel = 1:length(univariateModelNames)
        jModelName = univariateModelNames{jModel};
        disp([num2str(jModel), ' out of ', num2str(length(univariateModelNames)), ' (', jModelName, ')'])
        jOneStepPredictors = UnivariateOutput.(jModelName).predictors;
        IPT_tests.univariate.singleModel.(jModelName) = ...
            do_IPT_tests(jOneStepPredictors, jModelName, ...
            data, windowSize, distributions, UnivariateOutput.(jModelName));
    end
    save([baseDirectory,'IPT_tests.mat'],'IPT_tests')
end

%  ------------------------ multivariate - average model -----------------------------------------------------
if 0
    distributions = {'t7','normal'};
    for jStrategy = 1:length(strategyNames)
        jStrategyName = strategyNames{jStrategy};
        disp([num2str(jStrategy), ' out of ', num2str(length(strategyNames)), ' (', jStrategyName, ')'])
        IPT_tests.multivariate.averageModel.(jStrategyName) = ...
            do_IPT_tests_averageModel_multivariate(OneStepPredictors, SampleLikelihoods, Strategies.(jStrategyName), ...
            data, windowSize, distributions);
    end
    save([baseDirectory,'IPT_tests.mat'],'IPT_tests')
end

%  ------------------------ univariate - average model -----------------------------------------------------
if 0
    [UnivariateStrategies, univariateStrategyNames] = listOfStrategies(UnivariateModels);
    distributions = {'t7','normal'};
    for jStrategy = 1:length(univariateStrategyNames)
        jStrategyName = strategyNames{jStrategy};
        disp([num2str(jStrategy), ' out of ', num2str(length(univariateStrategyNames)), ' (', jStrategyName, ')'])
        IPT_tests.univariate.averageModel.(jStrategyName) = ...
            do_IPT_tests_averageModel_univariate(UnivariateOutput, UnivariateStrategies.(jStrategyName), ...
            data, windowSize, distributions);
    end
    save([baseDirectory,'IPT_tests.mat'],'IPT_tests')
end