function [omegaStar, isBinding, VaR, utility, psi] = meanVariancePortfolio_average(Sigma, mu, alpha, DoF, maximumLoss, delta, weights)

nModels = size(Sigma,3);
N = size(Sigma,1);

if ~exist('weights','var')
    weights = ones(nModels,1)/nModels; % (default: equal weights)
end

muBar = zeros(N,1);
for j = 1:nModels
    muBar = muBar + weights(j)*mu(:,j);
end

SigmaBar = zeros(N,N);
for j = 1:nModels
    SigmaBar = SigmaBar + weights(j)*Sigma(:,:,j) + weights(j)*((mu(:,j)-muBar)*(mu(:,j)-muBar)');
end


% ----------------------------------------------------------------------------------------------------------
% Compute the unconstrained solution to the mean-variance problem and see if it satisfies the VaR constraint
% ----------------------------------------------------------------------------------------------------------
omega = 1/delta*inv(SigmaBar)*muBar;
VaR = compute_excessVaR(omega, mu, Sigma, maximumLoss, weights, alpha, DoF);
if VaR < 0
    isBinding = 0;
    omegaStar = omega;
    psi = 0;
else
    isBinding = 1;
    %     figure(1), close(1), figure(1)
    % ------------------------------------------------------------------------------
    % proceed with the secant method
    % ------------------------------------------------------------------------------
    epsilon = 10^(-5);
    omega_high = omega / 2; % (some starting value moving towards the constraint)
    [G, g] = compute_Gs(omega_high, mu, Sigma, maximumLoss, weights, DoF);
    psi_low = 0;
    psi_high = compute_psi(omega_high, muBar, SigmaBar, delta, g, G);
    omega_low = omega;
    VaR_high = compute_excessVaR(omega_high, mu, Sigma, maximumLoss, weights, alpha, DoF);
    VaR_low = compute_excessVaR(omega_low, mu, Sigma, maximumLoss, weights, alpha, DoF);

    notYetConverged = 1;
    nIterations = 0;
    while notYetConverged
        omega_sec = mean([omega_high, omega_low],2);
        psi_sec = psi_high - VaR_high*(psi_high - psi_low)/(VaR_high - VaR_low);
        nRecursions = 100;
        omegas = nan(length(omega_sec),nRecursions);
        for j = 1:nRecursions
            [G, g] = compute_Gs(omega_sec, mu, Sigma, maximumLoss, weights, DoF);
            omega_sec = (delta*SigmaBar + psi_sec*G) \ (muBar + psi_sec*g);
            omegas(:,j) = omega_sec;
        end
        VaR_sec = compute_excessVaR(omega_sec, mu, Sigma, maximumLoss, weights, alpha, DoF);
        if sign(VaR_sec) >= 0
            delta_psi = abs(psi_low-psi_sec);
            delta_omega = sqrt(sum(omega_low-omega_sec).^2);
            psi_low = psi_sec;
            omega_low = omega_sec;
            VaR_low = VaR_sec;
        elseif sign(VaR_sec) < 0
            delta_psi = abs(psi_high-psi_sec);
            delta_omega = sqrt(sum(omega_high-omega_sec).^2);
            psi_high = psi_sec;
            omega_high = omega_sec;
            VaR_high = VaR_sec;
        end
        notYetConverged = abs(VaR_sec) > epsilon;% || sqrt(sum(delta_omega.^2)) > epsilon*100;
        nIterations = nIterations + 1;
    end
    omegaStar = omega_sec;
    VaR = VaR_sec;
    psi = psi_sec;
end

utility = omegaStar'*muBar - delta/2*omegaStar'*SigmaBar*omegaStar;

% --------------------------------------------------------------------------------------------------

function [G, g] = compute_Gs(omega, mu, Sigma, maximumLoss, weights, DoF)

[N,nModels] = size(mu);
G = zeros(N,N);
g = zeros(N,1);

for j = 1:nModels
    switch DoF(j)
        case inf
            phi_j = normpdf(((-omega'*mu(:,j)-maximumLoss)/sqrt(omega'*Sigma(:,:,j)*omega)),0,1);
            G = G + weights(j)*phi_j*(omega'*mu(:,j)+maximumLoss)*Sigma(:,:,j)/(omega'*Sigma(:,:,j)*omega)^1.5;
            g = g + weights(j)*phi_j*mu(:,j)/(omega'*Sigma(:,:,j)*omega)^0.5;
        otherwise
            phi_j = tpdf(((-omega'*mu(:,j)-maximumLoss)/sqrt((DoF(j)-2)/DoF(j)*omega'*Sigma(:,:,j)*omega)),DoF(j));
            G = G + weights(j)*phi_j*(omega'*mu(:,j)+maximumLoss)*Sigma(:,:,j)/sqrt((DoF(j)-2)/DoF(j))/(omega'*Sigma(:,:,j)*omega)^1.5;
            g = g + weights(j)*phi_j*mu(:,j)/((DoF(j)-2)/DoF(j)*omega'*Sigma(:,:,j)*omega)^0.5;
    end

end

% --------------------------------------------------------------------------------------------------

function  psi = compute_psi(omega, muBar, SigmaBar, delta, g, G)

psi = -(omega'*muBar-delta*omega'*SigmaBar*omega) / (omega'*g - omega'*G*omega);

% --------------------------------------------------------------------------------------------------

function excessVaR = compute_excessVaR(omega, mu, Sigma, maximumLoss, weights, alpha, DoF)

nModels = size(mu,2);
VaR = 0;
for j = 1:nModels
    if isinf(DoF(j))
        VaR = VaR + weights(j)*normcdf((-omega'*mu(:,j)-maximumLoss)/sqrt(omega'*Sigma(:,:,j)*omega),0,1);
    else
        VaR = VaR + weights(j)*tcdf((-omega'*mu(:,j)-maximumLoss)/sqrt((DoF(j)-2)/DoF(j)*omega'*Sigma(:,:,j)*omega),DoF(j));
    end
end
excessVaR = VaR - alpha;
