function [omegaStar, isBinding, VaR, exitFlag, utility, psi] = meanVariancePortfolio_average_Newton(Sigma, mu, alpha, ...
    DoF, maximumLoss, delta, weights, constrainPsi)

if ~exist('constrainPsi','var')
    constrainPsi = 0;
end

nModels = size(Sigma,3);
N = size(Sigma,1);

if ~exist('weights','var')
    weights = ones(nModels,1)/nModels; % (default: equal weights)
end

muBar = zeros(N,1);
for j = 1:nModels
    muBar = muBar + weights(j)*mu(:,j);
end

SigmaBar = zeros(N,N);
for j = 1:nModels
    SigmaBar = SigmaBar + weights(j)*Sigma(:,:,j) + weights(j)*((mu(:,j)-muBar)*(mu(:,j)-muBar)');
end


% ----------------------------------------------------------------------------------------------------------
% Compute the unconstrained solution to the mean-variance problem and see if it satisfies the VaR constraint
% ----------------------------------------------------------------------------------------------------------
omega = 1/delta*inv(SigmaBar)*muBar;
omegaUnconstrained = omega;
psi = 0;
VaR = compute_excessVaR(omega, mu, Sigma, maximumLoss, weights, alpha, DoF);
if VaR < 0
    isBinding = 0;
    omegaStar = omega;
    exitFlag = 1;
else
    isBinding = 1;
    %     figure(1), close(1), figure(1)
    % ------------------------------------------------------------------------------
    % proceed with the Newton method
    % ------------------------------------------------------------------------------
    epsilon = 10^(-5);
    while VaR > 0
        omega = omega / 1.01; % (some starting value moving towards the constraint)
        VaR = compute_excessVaR(omega, mu, Sigma, maximumLoss, weights, alpha, DoF);
    end
    [G, g] = compute_Gs(omega, mu, Sigma, maximumLoss, weights, DoF);
    psi = compute_psi(omega, muBar, SigmaBar, delta, g, G);
    x = [omega; psi];

    notYetConverged = 1;
    nIterations = 0;
    while notYetConverged

        z   = compute_z(omega, mu, Sigma, maximumLoss, DoF);
        z_w = compute_z_w(omega, mu, Sigma, maximumLoss, DoF);
        z_ww = compute_z_ww(omega, mu, Sigma, maximumLoss, DoF);
        pdf_z = compute_pdf_z(z, DoF);
        cdf_z = compute_cdf_z(z, DoF);
        pdf_z_prime = compute_pdf_z_prime(z, DoF);

        f_1 = muBar - delta*SigmaBar*omega;
        f_2 = alpha;

        j_11 = - delta*SigmaBar;
        j_12 = 0;
        j_21 = 0;
        j_22 = 0;

        for j = 1:nModels
            f_1 = f_1 + psi*weights(j)*pdf_z(j)*z_w(:,j);
            f_2 = f_2 - weights(j)*cdf_z(j);

            j_11 = j_11 - psi*weights(j)*(pdf_z_prime(j)*z_w(:,j)*z_w(:,j)' + pdf_z(j)*z_ww(:,:,j));
            j_12 = j_12 - weights(j)*pdf_z(j)*z_w(:,j);
            j_21 = j_21 - weights(j)*pdf_z(j)*z_w(:,j)';
        end

        f = [f_1; f_2];
        J = [j_11, j_12; j_21, j_22];
        xOld = x;
        deltaX = - J \ f;
        x = xOld + deltaX;
        omega = x(1:N);
        if constrainPsi
            [G, g] = compute_Gs(omega, mu, Sigma, maximumLoss, weights, DoF);
            x(N+1) = compute_psi(omega, muBar, SigmaBar, delta, g, G);
            deltaX = x - xOld;
        end
        psi = x(N+1);

        VaR = compute_excessVaR(omega, mu, Sigma, maximumLoss, weights, alpha, DoF);
        notYetConverged = abs(VaR) > epsilon || sqrt(sum(deltaX.^2)) > epsilon*100;

        nIterations = nIterations + 1;
        if nIterations > 200
            notYetConverged = 0;
        end
    end
    omegaStar = omega;
    if nIterations > 200 || any(isnan(omegaStar))
        exitFlag = 0;
    else
        exitFlag = 1;
    end
end

utility = omegaStar'*muBar - delta/2*omegaStar'*SigmaBar*omegaStar;

[G, g] = compute_Gs(omegaStar, mu, Sigma, maximumLoss, weights, DoF);
psiStar = compute_psi(omegaStar, muBar, SigmaBar, delta, g, G);


% --------------------------------------------------------------------------------------------------

function z = compute_z(omega, mu, Sigma, maximumLoss, DoF)

[N,nModels] = size(mu);
z = nan(nModels,1);

for j = 1:nModels
    switch DoF(j)
        case inf
            DoF_adjustment = 1;
        otherwise
            DoF_adjustment = sqrt((DoF(j) - 2)/DoF(j));
    end
    z(j) = (-omega'*mu(:,j)-maximumLoss)/sqrt(omega'*Sigma(:,:,j)*omega);
    z(j) = z(j) / DoF_adjustment;
end

% --------------------------------------------------------------------------------------------------

function z_w = compute_z_w(omega, mu, Sigma, maximumLoss, DoF)

[N,nModels] = size(mu);
z_w = nan(N,nModels);

for j = 1:nModels
    switch DoF(j)
        case inf
            DoF_adjustment = 1;
        otherwise
            DoF_adjustment = sqrt((DoF(j) - 2)/DoF(j));
    end
    z_w(:,j) = -mu(:,j) / sqrt(omega'*Sigma(:,:,j)*omega) + (omega'*mu(:,j)+maximumLoss)/(omega'*Sigma(:,:,j)*omega)^(3/2) ...
        * Sigma(:,:,j)*omega;
    z_w(:,j) = z_w(:,j) / DoF_adjustment;
end

% --------------------------------------------------------------------------------------------------

function z_ww = compute_z_ww(omega, mu, Sigma, maximumLoss, DoF)

[N,nModels] = size(mu);
z_ww = nan(N,N,nModels);

for j = 1:nModels
    switch DoF(j)
        case inf
            DoF_adjustment = 1;
        otherwise
            DoF_adjustment = sqrt((DoF(j) - 2)/DoF(j));
    end
    quadraticTerm = omega'*Sigma(:,:,j)*omega;

    z_ww(:,:,j) = quadraticTerm^(-3/2)*(mu(:,j)*omega'*Sigma(:,:,j) ...
        + Sigma(:,:,j)*omega*mu(:,j)' + (omega'*mu(:,j) + maximumLoss)*Sigma(:,:,j)) ...
        - 3*(omega'*mu(:,j) + maximumLoss) / quadraticTerm^(5/2) * Sigma(:,:,j)*omega*omega'*Sigma(:,:,j);

    z_ww(:,:,j) = z_ww(:,:,j) / DoF_adjustment;
end

% --------------------------------------------------------------------------------------------------

function pdf_z = compute_pdf_z(z, DoF)

pdf_z = nan(size(z));
for j = 1:length(z)
    switch DoF(j)
        case inf
            pdf_z(j) = normpdf(z(j));
        otherwise
            pdf_z(j) = tpdf(z(j),DoF(j));
    end
end

% --------------------------------------------------------------------------------------------------

function cdf_z = compute_cdf_z(z, DoF)

cdf_z = nan(size(z));
for j = 1:length(z)
    switch DoF(j)
        case inf
            cdf_z(j) = normcdf(z(j));
        otherwise
            cdf_z(j) = tcdf(z(j),DoF(j));
    end
end

% --------------------------------------------------------------------------------------------------

function pdf_z_prime = compute_pdf_z_prime(z, DoF)

pdf_z_prime = nan(size(z));
for j = 1:length(z)
    switch DoF(j)
        case inf
            pdf_z_prime(j) = 1/sqrt(2*pi)*exp(-z(j)^2/2)*(-z(j));
        otherwise
            term = exp(gammaln((DoF(j)+1)/2) - gammaln(DoF(j)/2)) / sqrt(DoF(j)*pi);
            pdf_z_prime(j) = term * (-DoF(j)-1)/2 * 2*z(j) / DoF(j) * ((1+z(j)^2)/DoF(j))^(-(DoF(j)+1)/2-1);
    end
end

% --------------------------------------------------------------------------------------------------

function [G, g] = compute_Gs(omega, mu, Sigma, maximumLoss, weights, DoF)

[N,nModels] = size(mu);
G = zeros(N,N);
g = zeros(N,1);

for j = 1:nModels
    switch DoF(j)
        case inf
            phi_j = normpdf(((-omega'*mu(:,j)-maximumLoss)/sqrt(omega'*Sigma(:,:,j)*omega)),0,1);
            G = G + weights(j)*phi_j*(omega'*mu(:,j)+maximumLoss)*Sigma(:,:,j)/(omega'*Sigma(:,:,j)*omega)^1.5;
            g = g + weights(j)*phi_j*mu(:,j)/(omega'*Sigma(:,:,j)*omega)^0.5;
        otherwise
            phi_j = tpdf(((-omega'*mu(:,j)-maximumLoss)/sqrt((DoF(j)-2)/DoF(j)*omega'*Sigma(:,:,j)*omega)),DoF(j));
            G = G + weights(j)*phi_j*(omega'*mu(:,j)+maximumLoss)*Sigma(:,:,j)/sqrt((DoF(j)-2)/DoF(j))/(omega'*Sigma(:,:,j)*omega)^1.5;
            g = g + weights(j)*phi_j*mu(:,j)/((DoF(j)-2)/DoF(j)*omega'*Sigma(:,:,j)*omega)^0.5;
    end

end

% --------------------------------------------------------------------------------------------------

function  psi = compute_psi(omega, muBar, SigmaBar, delta, g, G)

psi = -(omega'*muBar-delta*omega'*SigmaBar*omega) / (omega'*g - omega'*G*omega);

% --------------------------------------------------------------------------------------------------

function excessVaR = compute_excessVaR(omega, mu, Sigma, maximumLoss, weights, alpha, DoF)

nModels = size(mu,2);
VaR = 0;
for j = 1:nModels
    if isinf(DoF(j))
        VaR = VaR + weights(j)*normcdf((-omega'*mu(:,j)-maximumLoss)/sqrt(omega'*Sigma(:,:,j)*omega),0,1);
    else
        VaR = VaR + weights(j)*tcdf((-omega'*mu(:,j)-maximumLoss)/sqrt((DoF(j)-2)/DoF(j)*omega'*Sigma(:,:,j)*omega),DoF(j));
    end
end
excessVaR = VaR - alpha;
