function [c, ceq] = ADCC_nonlincon(params, stdresid, P, Q, epsilon)

% PURPOSE:
%        Nonlinear constraint function for use in the ADCC_MVGARCH estimation 
% 
% USAGE:
%        [c, ceq] = ADCC_nonlincon(params, stdresid, P, Q)
% 
% INPUTS:
%    params      - A 2*P+Q by 1 vector of parameters of the form [dccPparameters; dccQparameters, adccPparameters]
%    stdresid    - A matrix, t x k of residuals standardized by their conditional standard deviation
%    P           - The innovation order of the ADCC Garch process
%    Q           - The AR order of the ADCC estimator
% 
% OUTPUTS:
%    c              - The inequality constraint
%    ceq            - The equality constraint

% 
% COMMENTS:
% 
% 
% Author: Christoph Schleicher

[t,k] = size(stdresid);
a          = params(1:P);       % ARCH for all residuals
a_negative = params(P+1:2*P);   % ARCH for negative residuals
b = params(2*P+1:2*P+Q);        % GARCH
sumA = eye(k)*sum(a);
sumA_negative = eye(k)*sum(a_negative);
sumB = eye(k)*sum(b);

% Compute Qbar, the correlation matrix of the standardized residuals,
% and Nbar, the correlation matrix of the standardized residuals,
% conditional on the standardized residuals being negative
Qbar = cov(stdresid);
Nbar_negative = cov(stdresid.*(stdresid < 0));

% Next compute Qt
m = max(P,Q);
Qt = zeros(k,k,t+m);
Rt = zeros(k,k,t+m);
Qinitial = Qbar * (eye(k) - sumA - sumB) - Nbar_negative * sumA_negative;

x_1 = eig(Qinitial);
x_1 = x_1 / abs(max(x_1));
c = min(x_1)*(-1) + epsilon;
ceq = 0;
