function parameters = estimate_ADCC(data, dccP, dccQ, archP, garchQ, nTries, verbose, startingValues)

%  --------------------------------------------------------------------------------------------
%  PURPOSE: Estimates a multivariate GARCH model with asymmetric ARCH terms using the DCC
%           estimator of Engle and Sheppard
%  --------------------------------------------------------------------------------------------
%  USAGE: parameters = estimate_ADCC(data, dccP, dccQ, archP, garchQ, nTries, verbose)
%  where: series    = an m x T dimensional series of innovations
%         dccP      = the lag length of the innovation term in the DCC estimator
%         dccQ      = the lag length of the lagged correlation matrices in the DCC estimator
%         archP and garchP = parameters of univariate GARCH model
%         nTries    = number of times each univariate GARCH model is estimated for different
%                     starting values
%         verbose   = 0, 1, or 2
%  --------------------------------------------------------------------------------------------
%  RETURNS: a structure parameters with fields
%               GJR (an m-dimensional cell array with the univariate GJR parameters)
%               correlation (contains the parameters for the multivariate component)
%  --------------------------------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca) based on code by Kevin Sheppard
%  --------------------------------------------------------------------------------------------

%  --------------------------------------------------------------------------------------------
%  (1) checking the inputs
%  --------------------------------------------------------------------------------------------

[t,k]=size(data);

if k<2
    error('Must have at least 2 data series')
end

if length(dccP)~=length(dccQ) || length(dccP)~=1
    error('dccP and dccQ must be scalars')
end

if ~(isempty(archP) || length(archP)==1 || length(archP)==k)
    error('Wrong size for archP')
end

if ~(isempty(garchQ) || length(garchQ)==1 || length(garchQ)==k)
    error('Wrong size for garchQ')
end

if isempty(archP)
    archP=ones(1,k);
elseif length(archP)==1
    archP=ones(1,k)*archP;
end

if isempty(garchQ)
    garchQ=ones(1,k);
elseif length(garchQ)==1
    garchQ=ones(1,k)*garchQ;
end

%  --------------------------------------------------------------------------------------------
%  (2) estimate the univariate GARCH models
%  --------------------------------------------------------------------------------------------

standardizedResiduals = nan(size(data));
univariate = cell(k,1);
for i=1:k
    fprintf(1,'Estimating GJR model for Series %d\n',i)

    options = optimset('fmincon');
    options = optimset(options, ...
        'Display','off', ...
        'Diagnostics','off', ...
        'MaxFunEvals', 1000*max(archP(i)+garchQ(i)+1), ...
        'MaxIter',1000*max(archP(i)+garchQ(i)+1), ...
        'LargeScale','off', ...
        'MaxSQPIter',1000);

    if exist('startingValues','var')
        iStartingValues = startingValues.GJR{i};
    else
        iStartingValues = [];
    end
    [univariate{i}.parameters, univariate{i}.likelihood, univariate{i}.stderrors, univariate{i}.ht] ...
        = estimate_GJR(data(:,i), archP(i), garchQ(i), 'NORMAL', nTries, verbose, options, iStartingValues);

    standardizedResiduals(:,i) = data(:,i)./sqrt(univariate{i}.ht);
end

%  --------------------------------------------------------------------------------------------
%  (3) estimate the conditional correlation matrix
%  --------------------------------------------------------------------------------------------

fprintf(1,'\nEstimating the DCC model\n\n')

options=optimset('fmincon');
options  =  optimset(options , 'Display'     , 'off');
options  =  optimset(options , 'Diagnostics' , 'off');
options  =  optimset(options , 'LevenbergMarquardt' , 'on');
options  =  optimset(options , 'LargeScale'  , 'off');

if exist('startingValues','var')
    dccstarting = startingValues.correlation;
else
    dccstarting = [ones(1,dccP)*.001/dccP,  zeros(1,dccP), ones(1,dccQ)*.001/dccQ]';
end
dccstarting_2 = [ones(1,dccP)*.001/dccP,  zeros(1,dccP), ones(1,dccQ)*.001/dccQ]';

A = ones(1,2*dccP+dccQ);
b = 1;
lb = zeros(2*dccP+dccQ,1)+2*options.TolCon;
ub = [ones(dccP,1)*0.99; ones(dccP,1)*0.9; ones(dccQ,1)*0.999];

epsilon = 10^(-3);
exitFlag = 0;
count = 0;
while exitFlag < 1 && count <= 10
    if epsilon < 10^(-3)
        disp(['epsilon = ', num2str(epsilon)])
    end
    [dccparameters, negativeLikelihood, exitFlag] = fmincon('ADCC_likelihood', dccstarting, A, b, ...
        [], [], lb, ub, 'ADCC_nonlincon', options, standardizedResiduals, dccP, dccQ, epsilon);
    if count == 1
        dccstarting = dccstarting_2;
    else
        epsilon = epsilon/2;
    end
    count = count + 1;
end

if exitFlag < 1
    disp('optimization did not finish successfully')
    keyboard
end


%  --------------------------------------------------------------------------------------------
%  (4) set up the output structure
%  --------------------------------------------------------------------------------------------

parameters = struct;
for i = 1:k
    parameters.GJR{i} = univariate{i}.parameters;
end
parameters.correlation = dccparameters;