function Sigma = evaluate_ADCC(series, dccP, dccQ, p, q, parameters)

%  ---------------------------------------------------------------------
%  PURPOSE: computes a covariance estimate using the asymmetric
%           dynamic condtional correlation (ADCC model)
%  ---------------------------------------------------------------------
%  USAGE: [Sigma, Ht, Rt] = evaluate_ADCC(series, dccP, dccQ, p, q, estimatedParameters)
%  where: series    = an m x T dimensional series of innovations
%         p, q      = parameters of univariate GARCH(p, q)
%         estimatedParameters.GARCH = an m-dimensional structure with the
%                       GARCH parameters (mean, ARCH, GARCH)
%         estimatedParameters.correlationMatrix = the (fixed) correlation
%                       matrix
% ---------------------------------------------------------------------
%  RETURNS: the covariance matrix
% ---------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca)
% ---------------------------------------------------------------------

[T, m] = size(series);

H = nan(T+1,m);
for j = 1:m
    H(:,j) = evaluate_GJR(series(:,j), parameters.GJR{j}, p, q);
end
Hstd = H.^(0.5);
standardizedSeries = [series; zeros(1,m)] ./ Hstd;

[loglikelihood, Rt] = ADCC_likelihood(parameters.correlation, standardizedSeries, dccP, dccQ);

Sigma = zeros(m,m,T+1);
for t = 1:T+1
    Sigma(:,:,t) = diag(Hstd(t,:))*Rt(:,:,t)*diag(Hstd(t,:));
end