function Sigma = evaluate_CCC(series, p, q, estimatedParameters)

%  ---------------------------------------------------------------------
%  PURPOSE: computes a covariance estimate using Bollerslev's Constant
%           Conditional Correlation (CCC) model
%  ---------------------------------------------------------------------
%  USAGE: Sigma = evaluate_CCC(series, p, q, estimatedParameters)
%  where: series    = an m x T dimensional series of innovations
%         p, q      = parameters of univariate GARCH(p, q)
%         estimatedParameters.GARCH = an m-dimensional structure with the
%                       GARCH parameters (mean, ARCH, GARCH)
%         estimatedParameters.correlationMatrix = the (fixed) correlation
%                       matrix
% ---------------------------------------------------------------------
%  RETURNS: the covariance matrix (an m x m x T+1 array)
% ---------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca)
% ---------------------------------------------------------------------

[T, m] = size(series);

% ---------------------------------------------------------------------
% (1) predict individual GARCH(p,q) models for the diagonal matrix D
% ---------------------------------------------------------------------
H = nan(T+1,m);
for j = 1:m
     H(:,j) = evaluate_GARCH(series(:,j), estimatedParameters.GARCH{j}, p, q);
end

% ---------------------------------------------------------------------
% (2) construct the covariance matrix as D*R*D
% ---------------------------------------------------------------------
R = estimatedParameters.correlation;

Sigma = zeros(m,m,T+1);
Hstd = H.^(0.5);
for t = 1:T+1
    Sigma(:,:,t) = diag(Hstd(t,:))*R*diag(Hstd(t,:));
end