function parameters = estimate_DCC(data, dccP, dccQ, archP, garchQ, nTries, verbose, startingValues)

%  --------------------------------------------------------------------------------------------
%  PURPOSE: Estimates a multivariate GARCH model using the DCC estimator of Engle and Sheppard
%  --------------------------------------------------------------------------------------------
%  USAGE: parameters = estimate_DCC(data, dccP, dccQ, archP, garchQ, nTries, verbose)
%  where: series    = an m x T dimensional series of innovations
%         dccP      = the lag length of the innovation term in the DCC estimator
%         dccQ      = the lag length of the lagged correlation matrices in the DCC estimator
%         archP and garchP = parameters of univariate GARCH model
%         nTries    = number of times each univariate GARCH model is estimated for different
%                     starting values
%         verbose   = 0, 1, or 2
%  --------------------------------------------------------------------------------------------
%  RETURNS: a structure parameters with fields
%               GARCH (an m-dimensional cell array with the univariate GARCH parameters)
%               correlation (contains the parameters for the multivariate component)
%  --------------------------------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca) based on code by Kevin Sheppard
%  --------------------------------------------------------------------------------------------

%  --------------------------------------------------------------------------------------------
%  (1) checking the inputs
%  --------------------------------------------------------------------------------------------

[t,k] = size(data);
if k < 2
    error('Must have at least 2 data series')
end

if length(dccP)~=length(dccQ) || length(dccP)~=1
    error('dccP and dccQ must be scalars')
end

if ~(isempty(archP) || length(archP)==1 || length(archP)==k)
    error('Wrong size for archP')
end

if ~(isempty(garchQ) || length(garchQ)==1 || length(garchQ)==k)
    error('Wrong size for garchQ')
end

if isempty(archP)
    archP = ones(1,k);
elseif length(archP)==1
    archP=ones(1,k)*archP;
end

if isempty(garchQ)
    garchQ=ones(1,k);
elseif length(garchQ)==1
    garchQ=ones(1,k)*garchQ;
end

%  --------------------------------------------------------------------------------------------
%  (2) estimate the univariate GARCH models
%  --------------------------------------------------------------------------------------------

standardizedResiduals = nan(size(data));
univariate = cell(k,1);
for i = 1:k
    fprintf(1,'Estimating GARCH model for Series %d\n',i)
    options = optimset('fmincon');
    options = optimset(options, ...
        'Display','off', ...
        'Diagnostics','off', ...
        'MaxFunEvals', 1000*max(archP(i)+garchQ(i)+1), ...
        'MaxIter',1000*max(archP(i)+garchQ(i)+1), ...
        'LargeScale','off', ...
        'MaxSQPIter',1000);

    if exist('startingValues','var')
        iStartingValues = startingValues.GARCH{i};
    else
        iStartingValues = [];
    end
    [univariate{i}.parameters, univariate{i}.likelihood, univariate{i}.stderrors,univariate{i}.robustSE, univariate{i}.ht, univariate{i}.scores] ...
        = estimate_GARCH(data(:,i), archP(i), garchQ(i), 'NORMAL', nTries, verbose, options, iStartingValues);

    standardizedResiduals(:,i) = data(:,i)./sqrt(univariate{i}.ht);
end

%  --------------------------------------------------------------------------------------------
%  (3) estimate the conditional correlation matrix
%  --------------------------------------------------------------------------------------------

options=optimset('fmincon');
options  =  optimset(options , 'Display'     , 'off');
options  =  optimset(options , 'Diagnostics' , 'off');
options  =  optimset(options , 'LevenbergMarquardt' , 'on');
options  =  optimset(options , 'LargeScale'  , 'off');

if exist('startingValues','var')
    dccstarting = startingValues.correlation;
else
    dccstarting = [ones(1,dccP)*.01/dccP, ones(1,dccQ)*.97/dccQ];
end


fprintf(1,'\nEstimating the DCC model\n\n')

A = ones(size(dccstarting));
b = 1-2*options.TolCon;
lb = zeros(size(dccstarting))+2*options.TolCon;

[dccparameters, dccllf, EXITFLAG, OUTPUT, LAMBDA, GRAD] = fmincon('dcc_mvgarch_likelihood', ...
    dccstarting, A, b, [], [], lb, [], [], options, standardizedResiduals, dccP, dccQ);

%  --------------------------------------------------------------------------------------------
%  (4) set up the output structure
%  --------------------------------------------------------------------------------------------

parameters = struct;
for i = 1:k
    parameters.GARCH{i} = univariate{i}.parameters;
end
parameters.correlation = dccparameters;