function [parameters, likelihood, stderrors, robustSE, ht, scores] = ...
    estimate_GARCH(data, p, q, errors, nTries, verbose, options, startingValues)

% PURPOSE: estimate a univariate GARCH model
%
% USAGE:
%     [parameters, likelihood, stderrors, robustSE, ht, scores] = ...
%               estimate_GARCH(data, p, q, errors, nTries, verbose, options, startingValues)
%
% INPUTS:
%     data: A single column of zero mean random data, normal or not for quasi likelihood
%
%     P: Non-negative, scalar integer representing a model order of the ARCH
%       process
%
%     Q: Positive, scalar integer representing a model order of the GARCH
%       process: Q is the number of lags of the lagged conditional variances included
%       Can be empty([]) for ARCH process
%
%     error:  The type of error being assumed, valid types are:
%            'NORMAL' - Gaussian Innovations
%            'STUDENTST' - T-distributed errors
%            'GED' - General Error Distribution
%
%     startingvalues: A (1+p+q) (plus 1 if STUDENTT OR GED is selected for the nu parameter) vector of starting vals.
%       If you do not provide, a naieve guess of 1/(2*max(p,q)+1) is used for the arch and garch parameters,
%       and omega is set to make the real unconditional variance equal
%       to the garch expectation of the expectation.
%
%     options: default options are below.  You can provide an options vector.  See HELP OPTIMSET
%
%      number of times the optimizer starts with different starting values
%
% OUTPUTS:
%     parameters : a [1+p+q X 1] column of parameters with omega, alpha1, alpha2, ..., alpha(p)
%                  beta1, beta2, ... beta(q)
%
%     likelihood = the loglikelihood evaluated at he parameters
%
%     robustSE = QuasiLikelihood std errors which are robust to some forms of misspecification(see White 94)
%
%     stderrors = the inverse analytical hessian, not for quasi maximum liklihood
%
%     ht = the estimated time varying VARIANCES
%
%     scores = The numberical scores(# fo params by t) for M testing
%
%
% COMMENTS:
%   GARCH(P,Q) the following(wrong) constratins are used(they are right for the (1,1) case or any Arch case
%     (1) Omega > 0
%     (2) Alpha(i) >= 0 for i = 1,2,...P
%     (3) Beta(i)  >= 0 for i = 1,2,...Q
%     (4) sum(Alpha(i) + Beta(j)) < 1 for i = 1,2,...P and j = 1,2,...Q
%     (5) nu>2 of Students T and nu>1 for GED
%
%   The time-conditional variance, H(t), of a GARCH(P,Q) process is modeled
%   as follows:
%
%     H(t) = Omega + Alpha(1)*r_{t-1}^2 + Alpha(2)*r_{t-2}^2 +...+ Alpha(P)*r_{t-p}^2+...
%                    Beta(1)*H(t-1)+ Beta(2)*H(t-2)+...+ Beta(Q)*H(t-q)
%
%   Default Options
%
%   options  =  optimset('fmincon');
%   options  =  optimset(options , 'TolFun'      , 1e-003);
%   options  =  optimset(options , 'Display'     , 'iter');
%   options  =  optimset(options , 'Diagnostics' , 'on');
%   options  =  optimset(options , 'LargeScale'  , 'off');
%   options  =  optimset(options , 'MaxFunEvals' , '400*numberOfVariables');
%
%
%  uses fFATTAILED_GARCHLIKELIHOOD and GARCHCORE.  You should MEX, mex 'path\garchcore.c', the MEX source
%  The included MEX is for R12 Windows and was compiled with VC++6. It gives a 10-15 times speed improvement
%
%
% Author: Christoph Schleicher (christoph@iam.ubc.ca) based on code by Kevin Sheppard

if ~exist('nTries','var'), nTries = 5; end
if ~exist('verbose','var'), verbose = 0; end
if ~exist('startingValues','var'), startingValues = []; end

if ~exist('options','var')
    options = optimset('fmincon');
    options = optimset(options, ...
        'Display','off', ...
        'Diagnostics','off', ...
        'MaxFunEvals', 1000*max(p+q+1), ...
        'MaxIter',1000*max(p+q+1), ...
        'LargeScale','off', ...
        'MaxSQPIter',1000);
end

if verbose > 1
    disp(' ')
    disp('-----------------------')
end
maxLikelihood = -inf;
rand('state',1) % initialize generator for uniform random numbers
for k = 1:nTries
    exitflag = 0;
    count = 0;
    while exitflag <= 0
        if ~isempty(startingValues) && k == 1
            omega = startingValues(1);
            alpha = startingValues(2:1+p);
            beta  = startingValues(2+p:1+p+q);
        else
            alpha  =  [.2; zeros(p-1,1)] + rand(p,1)*0.1/p;
            beta   =  [.5; zeros(q-1,1)] + rand(q,1)*0.1/q;
            omega  =  (1-(sum(alpha)+sum(beta)))*cov(data); % initialize at unconditional expectation
        end
        if strcmpi(errors,'STUDENTST')
            nu  = 30;
        elseif strcmpi(errors,'GED')
            nu = 1.7;
        else
            nu=[];
        end
        startingValues = [omega; alpha; beta; nu];
        [kParameters, kLikelihood, kStderrors, kRobustSE, kHt, kScores, exitflag] = ...
            fattailed_garch_cs(data, p, q, upper(errors), startingValues, options);
        if exitflag <= 0
            count = count + 1;
            if verbose > 0
                disp(['no convergence  (', num2str(count), ')'])
            end
            if count > 10
                error('no convergence after 100 different starting values')
            end
        end
    end
    if kLikelihood > maxLikelihood
        parameters = kParameters;
        likelihood = kLikelihood;
        stderrors = kStderrors;
        robustSE = kRobustSE;
        ht = kHt;
        scores = kScores;
        maxLikelihood = likelihood;
    end
    if verbose > 1
        disp([k, likelihood, maxLikelihood])
    end
end