function minusLogLikelihood = tGARCH_likelihood(parameters, series)

%  ---------------------------------------------------------------------
%  PURPOSE: computes the likelihood of a mean-reverting risk-metrics model
%  ---------------------------------------------------------------------
%  USAGE: logLikelihood = TDCC_likelihood(series, nLags, parameters)
%  where: parameters = an 2*m + 3 vector containing the parameters
%               lambda_1 (univariate GARCH, m-dimensional)
%               lambda_2 (univariate ARCH, m-dimenstional)
%               phi_1    (GARCH of correlation matrix, scalar)
%               phi_2    (ARCH of correlation matrix, scalar)
%               nu       the degrees of freedom of the multivariate t-distribution
%         series    = an m x T dimensional series of innovations
%         nLags     = number of lags for computation of the devolatized errors
% ---------------------------------------------------------------------
%  RETURNS: the (negative of the) log likelihood
% ---------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca)
% ---------------------------------------------------------------------

T = length(series);
% disp(parameters)

% ---------------------------------------------------------------------
% (1) recursively construct the matrix D2
% ---------------------------------------------------------------------

lambda_1 = real(parameters(1));
lambda_2 = real(parameters(2));
nu       = real(parameters(3));

sigmaBar2 = var(series,1);
mu_D2 = sigmaBar2*(1 - lambda_1 - lambda_2);
D2 = nan(T+1,1);
D2(1) = mu_D2;
for t = 2:T+1
    D2(t) = mu_D2 + lambda_1*D2(t-1) + lambda_2*series(t-1)^2;
end

% ---------------------------------------------------------------------
% (2) compute the log-likelihood
% ---------------------------------------------------------------------

logLikelihood_t = nan(T, 1);
for t = 1:T
    Sigma_t = D2(t);
    logLikelihood_t(t) = -1/2*log(pi) - 1/2*log(det(Sigma_t)) ...
        + log(gamma((1 + nu)/2) / gamma(nu/2)) - 1/2*log(nu-2) ...
        - (1 + nu)/2*log(1 + series(t)^2/Sigma_t / (nu - 2));
end

minusLogLikelihood = -sum(logLikelihood_t);