function Sigma = evaluate_GEWMA(series, n_0, p, q, nu_0, estimatedParameters)

%  ---------------------------------------------------------------------
%  PURPOSE: computes a covariance estimate using the generalized
%           Exponentially-Weighted Moving Average Model (GEWMA)
%  ---------------------------------------------------------------------
%  USAGE: Sigma = evaluate_GEWMA(series, n_0, p, q, nu_0)
%  where: series    = an m x T dimensional series of innovations
%         n_0       = window size for EWMA
%         p, q      = parameters of univariate GARCH(p, q)
%         nu_0      = decay factor of co-variances
%         estimatedParameters = an m-dimensional cell-array with GARCH(p,q)
%                     parameters for each of the m time-series (each
%                     parameter-vector consists of 1 + p + q parameters for
%                     the mean, the ARCH and the GARCH terms)
% ---------------------------------------------------------------------
%  RETURNS: the covariance matrix
% ---------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca)
% ---------------------------------------------------------------------

[T,N] = size(series);
Sigma = nan(N,N,T+1);

% ---------------------------------------------------------------------
% (1) predict individual GARCH(p,q) models for the diagonal matrix D
% ---------------------------------------------------------------------
D = zeros(N,N,T+1);
for j = 1:N
    condVar = evaluate_GARCH(series(:,j), estimatedParameters{j}, p, q);
    D(j,j,:) = sqrt(condVar);
end

% ---------------------------------------------------------------------
% (2) construct the correlation matrix R using the risk-metrics filter
% ---------------------------------------------------------------------

Sigma_riskMetrics = evaluate_EWMA(series, n_0, nu_0, nu_0);

% ---------------------------------------------------------------------
% (3) construct the covariance matrix as D*R*D
% ---------------------------------------------------------------------
for t = 1:T+1
    R = Sigma_riskMetrics(:,:,t) ./ (sqrt(diag(Sigma_riskMetrics(:,:,t)))*sqrt(diag(Sigma_riskMetrics(:,:,t)))');
    Sigma(:,:,t) = D(:,:,t)*R*D(:,:,t);
end