function [parameters, likelihood, stderrors, ht] = estimate_GJR(data, p, q, errors, nTries, verbose, options, startingValues)

% PURPOSE: estimates the asymmetric GARCH model of Glosten, Jagannathan, and Runkle (1993) ("On the relation between the 
%          expected value and the volatility of nominal excess return on stocks", Journal of Finance, 48, 1889-1801)
%
% USAGE:
%     [parameters, likelihood, stderrors, ht] = estimate_GJR(data, p, q, errors, nTries, verbose, options, startingValues)
%
% INPUTS:
%     data: A single column of zero mean random data, normal or not for quasi likelihood
%
%     P: Non-negative, scalar integer representing a model order of the ARCH
%       process
%
%     Q: Positive, scalar integer representing a model order of the GARCH
%       process: Q is the number of lags of the lagged conditional variances included
%       Can be empty([]) for ARCH process
%
%     error:  The type of error being assumed, valid types are:
%            'NORMAL' - Gaussian Innovations
%            'STUDENTST' - T-distributed errors
%            'GED' - General Error Distribution
%
%     nTries = number of times the optimization is iterated using different starting values
%
%     startingvals: A (1+p+q) (plus 1 if STUDENTT OR GED is selected for the nu parameter) vector of starting vals.
%       If you do not provide, a naieve guess of 1/(2*max(p,q)+1) is used for the arch and garch parameters,
%       and omega is set to make the real unconditional variance equal
%       to the garch expectation of the expectation.
%
%     options: default options are below.  You can provide an options vector.  See HELP OPTIMSET
%
% OUTPUTS:
%     parameters : a [1+p+q X 1] column of parameters with omega, alpha1, alpha2, ..., alpha(p)
%                  beta1, beta2, ... beta(q)
%
%     likelihood = the loglikelihood evaluated at he parameters
%
%     stderrors = the inverse analytical hessian, not for quasi maximum liklihood
%
%     ht = the estimated time varying VARIANCES
%
% Author: Christoph Schleicher (christoph@iam.ubc.ca) based on code by Kevin Sheppard

if ~exist('nTries','var'), nTries = 1; end
if ~exist('verbose','var'), verbose = 0; end
if ~exist('options','var'), options = []; end
if ~exist('startingValues','var'), startingValues = []; end

A = [
    0, ones(1,p), zeros(1,p), ones(1,q);
    0, zeros(1,p), ones(1,p), ones(1,q);
    ];
b = [1;1];

lb = [std(data)/100; zeros(2*p+q,1)];
ub = [10*var(data); ones(2*p+q,1)];

if verbose > 1
    disp(' ')
    disp('-----------------------')
end
maxLikelihood = -inf;
rand('state',1) % initialize generator for uniform random numbers
for k = 1:nTries
    exitflag = 0;
    count = 0;
    while exitflag <= 0
        if ~isempty(startingValues) && k == 1
            omega           = startingValues(1);
            alpha_positive  = startingValues(2:1+p);
            alpha_negative  = startingValues(2+p:1+2*p);
            beta            = startingValues(2+2*p:1+2*p+q);
        else
            alpha_positive  =  [.1; zeros(p-1,1)] + rand(p,1)*0.1-0.05;
            alpha_negative  =  alpha_positive;
            beta            =  [.7; zeros(q-1,1)] + rand(q,1)*0.1-0.05;
            omega           =  (1-(sum(alpha_positive)+sum(beta)))*cov(data); % initialize at unconditional expectation
        end
        startingValues = [omega; alpha_positive; alpha_negative; beta];
        
        stdEstimate =  std(data,1);  
        appendedData = [stdEstimate(ones(max(p,q),1)); data];  
        
        [kParameters, minusLikelihood, exitflag, output, lambda, grad, hessian] = ...
            fmincon('GJRLikelihood', startingValues, A, b, [], [], lb, ub, [], options, appendedData, p, q, errors);
        kLikelihood = -minusLikelihood;
        [LLF, kHt] = GJRLikelihood(kParameters, data, p, q, errors);
        if exitflag <= 0
            count = count + 1;
            if verbose > 0
                disp(['no convergence  (', num2str(count), ')'])
            end
            if count > 99
                error('no convergence after 100 different starting values')
            end
        end
    end
    if kLikelihood > maxLikelihood
        parameters = kParameters;
        likelihood = kLikelihood;
        stderrors = diag(inv(hessian));
        maxLikelihood = likelihood;
        ht = kHt;
    end
    if verbose > 1
        disp([k, likelihood, maxLikelihood])
    end
end