function condVar = evaluate_GJR(series, parameters, p, q)

%  ---------------------------------------------------------------------
%  PURPOSE: computes the one-step ahead variance forecast based on an
%           innovation series and parameters for a GJR(p,q) model
%  ---------------------------------------------------------------------
%  USAGE: conditionalVariance = garchpq_oneStepAheadVariance(series, parameters, p, q)
%  where: series        = a Tx1 time series of innovations
%         parameters    = a vector of dimension p+q+1 containing the mean,
%                         the ARCH and the GARCH parameters
%         p             = number of ARCH terms
%         q             = number of GARCH terms
% ---------------------------------------------------------------------
%  RETURNS: the conditional covariance matrix for t = T+1
% ---------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca)
% ---------------------------------------------------------------------

stdEstimate = std(series,1);
m = max(p,q);
series = [ones(m,1)*stdEstimate; series];
isPositive = series > 0;
isNegative = series < 0;
T = length(series);
condVar = nan(size(series));
condVar(1:m) = series(1)^2;
for t = (m + 1):T
   condVar(t) = parameters' * ...
       [1; ...
       isPositive(t-(1:p)).*series(t-(1:p)).^2; ...
       isNegative(t-(1:p)).*series(t-(1:p)).^2;  ...
       condVar(t-(1:q))];
end
condVar = condVar(m+1:T);

condVar1stepAhead = parameters' * ...
    [1; ...
    isPositive(end-(1:p)).*series(end-(1:p)+1).^2; ...
    isNegative(end-(1:p)).*series(end-(1:p)+1).^2;  ...
    condVar(end-(1:q)+1)];

condVar = [condVar; condVar1stepAhead];