function parameters = estimate_OGARCH(series, p, q, errors, nTries, verbose, options, startingValues)

% ---------------------------------------------------------------------------------
%  PURPOSE: estimates the parameters of the orthogonal GARCH model
% ---------------------------------------------------------------------------------
%  USAGE: parameters = estimate_OGARCH(series, p, q)
%  where: series    = an N x T dimensional series of innovations
%         p, q      = parameters of univariate GARCH(p, q)
%         errors    = 'NORMAL' - Gaussian Innovations
%                     'STUDENTST' - T-distributed errors
%                     'GED' - General Error Distribution
%         nTries    = number of different random starting values (default = 1)
%         verbose   = 0 (no output), 1, or 2
%         options   = a structure of options passed on to the optimization toolbox
% ---------------------------------------------------------------------------------
%  RETURNS: the estimated parameters
% ---------------------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca)
% ---------------------------------------------------------------------------------

[T,N] = size(series);

% ---------------------------------------------------------------------------------
% (1) standardize the innovations (rTilde)
% ---------------------------------------------------------------------------------

rTilde = (series - ones(T,1)*mean(series,1)) ./ (ones(T,1)*std(series,1));

% ---------------------------------------------------------------------------------
% (2) compute the sample covariance of rTilde, S
% ---------------------------------------------------------------------------------

S = 1/T * rTilde'*rTilde;

% ---------------------------------------------------------------------------------
% (3) compute the eigenvectors W and eigenvalues Lambda of S
% ---------------------------------------------------------------------------------

[W, Lambda] = eig(S); %#ok<NASGU>

% ---------------------------------------------------------------------------------
% (4) estimate the (univariate) GARCH parameters of the orthogonalized series r*W
% ---------------------------------------------------------------------------------

rW = rTilde*W;
parameters = cell(N,1);
if ~exist('options','var')
    options = set_options(p,q);
end
for j = 1:N
    if exist('startingValues','var')
        parameters{j} = estimate_GARCH(rW(:,j), p, q, errors, nTries, verbose, options, startingValues{j});
    else
        parameters{j} = estimate_GARCH(rW(:,j), p, q, errors, nTries, verbose, options);
    end
end


function options = set_options(p,q)
options = optimset('fmincon');
options = optimset(options, ...
    'Display','off', ...
    'Diagnostics','off', ...
    'MaxFunEvals', 1000*max(p+q+1), ...
    'MaxIter',1000*max(p+q+1), ...
    'LargeScale','off', ...
    'MaxSQPIter',1000);
