function Sigma = evaluate_OGARCH(series, p, q, estimatedParameters)

% ---------------------------------------------------------------------------------
%  PURPOSE: computes a covariance estimate using the orthogonal GARCH model
% ---------------------------------------------------------------------------------
%  USAGE: Sigma = evaluate_OGARCH(series, p, q, estimatedParameters)
%  where: series    = an N x T dimensional series of innovations
%         p, q      = parameters of univariate GARCH(p, q)
%         estimatedParameters = an N-dimensional structure with the
%                       GARCH parameters (mean, ARCH, GARCH)
% ---------------------------------------------------------------------------------
%  RETURNS: the covariance matrix
% ---------------------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca)
% ---------------------------------------------------------------------------------

[T,N] = size(series);

% ---------------------------------------------------------------------------------
% (1) standardize the innovations (rTilde)
% ---------------------------------------------------------------------------------

rTilde = (series - ones(T,1)*mean(series,1)) ./ (ones(T,1)*std(series,1));

% ---------------------------------------------------------------------------------
% (2) compute the sample covariance of rTilde, S
% ---------------------------------------------------------------------------------

S = 1/T * rTilde'*rTilde;

% ---------------------------------------------------------------------------------
% (3) compute the eigenvectors W and eigenvalues Lambda of S
% ---------------------------------------------------------------------------------

[W, Lambda] = eig(S); %#ok<NASGU>

% ---------------------------------------------------------------------------------
% (4) compute the matrix V (diagonal of sample standard deviation of 'series'
% ---------------------------------------------------------------------------------

V = diag(std(series,1));

% ---------------------------------------------------------------------------------
% (5) compute the one-step ahead GARCH variances of the orthogonalized series rTilde*W
% ---------------------------------------------------------------------------------

rW = rTilde*W;
Gamma = zeros(N,N,T+1);
for j = 1:N
    condVar = evaluate_GARCH(rW(:,j), estimatedParameters{j}, p, q);
    Gamma(j,j,:) = condVar;
end

% ---------------------------------------------------------------------------------
% (6) construct the conditional covariance matrix as Sigma = V*W*Gamma*W'*V
% ---------------------------------------------------------------------------------

Sigma = nan(N,N,T+1);
for t = 1:T+1
    Sigma(:,:,t) = V*W*Gamma(:,:,t)*W'*V;
end