function [minusLogLikelihood, Sigma] = TDCC_likelihood(parameters, r, p, mode)

%  ------------------------------------------------------------------------------------------------
%  PURPOSE: computes the negative of the log-likelihood and the conditional covariance matrices
%           of Pesaran & Pesaran's (2007) t-Dynamic Conditional Correlation (TDCC) model
%  ------------------------------------------------------------------------------------------------
%  USAGE: [minusLogLikelihood, Sigma] = TDCC_likelihood(parameters, r, p, mode)
%  where: parameters = a vector containing the estimated parameters
%           (1) mean-reverting case:
%                   lambda_1 (univariate GARCH, m-dimensional)
%                   lambda_2 (univariate ARCH, m-dimensional)
%                   phi_1    (GARCH of correlation matrix, scalar)
%                   phi_2    (ARCH of correlation matrix, scalar)
%                   nu     (degrees of freedom of the multivariate t-distribution, scalar)
%           (2) non-mean reverting case:
%                   lambda   (univariate ARCH, m-dimensional)
%                   phi      (ARCH of correlation matrix, scalar)
%                   nu     (degrees of freedom of the multivariate t-distribution, scalar)
%         r         = an m x T dimensional series of innovations
%         p         = number of lags for computation of the devolatized errors
%         mode      = 'mean-reverting' or 'non-mean-reverting'
% ------------------------------------------------------------------------------------------------
%  RETURNS: minusLogLikelihood, the (negative of the) log likelihood 
%           Sigma, an (m x m x T+1) array containing the conditional covariances 
% ------------------------------------------------------------------------------------------------
%  REFERENCE: B. Pesaran and M.H. Pesaran (2007) "Modelling Volatilities and
%       Conditional Correlations in Futures Markets with a Multivariate
%       t Distribution", mimeo.
% ------------------------------------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca)
% ------------------------------------------------------------------------------------------------

[T, m] = size(r);

% ------------------------------------------------------------------------------------------------
% (1) construct the devolatized returns rTilde by dividing r by the
%     nLag standard deviation
% ------------------------------------------------------------------------------------------------

rTilde = nan(size(r));
for j = 1:m
    sigmaTilde = sqrt(filter(ones(p,1)/p, 1, r(:,j).^2));
    warning('off','MATLAB:divideByZero')
    rTilde(:,j) = r(:,j) ./ sigmaTilde;
    warning('on','MATLAB:divideByZero')
    % --------------------------------------------------------------------
    %     NB: this is the same (but faster) as doing the following loop
    %         r2 = r.^2;
    %         for t = p:length(rTilde)
    %             rTilde(t,j) = r(t,j) / sqrt(mean(r2(t-p+1:t,j)));
    %         end
    % --------------------------------------------------------------------
end
rTilde(1:p-1,:) = nan;

% ------------------------------------------------------------------------------------------------
% (2) set up the parameters Lambda, phi, and nu
% ------------------------------------------------------------------------------------------------

switch mode
    case 'mean-reverting'
        Lambda_1 = diag(real(parameters(1:m)));
        Lambda_2 = diag(real(parameters(m+1:2*m)));
        if m > 1
            phi_1 = real(parameters(2*m+1));
            phi_2 = real(parameters(2*m+2));
            nu    = real(parameters(2*m+3));
        else
            phi_1 = 1;
            phi_2 = 0;
            nu    = real(parameters(2*m+1));
        end
    case 'non-mean-reverting'
        Lambda_1 = diag(real(parameters(1:m)));
        Lambda_2 = eye(m) - Lambda_1;
        if m > 1
            phi_1 = real(parameters(m+1));
            phi_2 = 1 - phi_1;
            nu    = real(parameters(m+2));
        else
            phi_1 = 1;
            phi_2 = 0;
            nu    = real(parameters(m+1));
        end
end

% ------------------------------------------------------------------------------------------------
% (3) initialize some arrays
% ------------------------------------------------------------------------------------------------

D2 = nan(m, m, T+1);
R  = nan(m, m, T+1);
D  = nan(m, m, T+1);
Q  = nan(m, m, T+1);
Sigma = nan(m, m, T+1);

% ------------------------------------------------------------------------------------------------
% (4) initialize the matrices D2 (=D*D) and Q using the first p obs
% ------------------------------------------------------------------------------------------------

D2(:,:,p) = diag(mean(r(1:p,:).^2,1));
D(:,:,p) = sqrt(D2(:,:,p));

SigmaBar = diag(mean(r(p+1:end,:).^2));
QBar = (r(p+1:end,:)'*r(p+1:end,:)) ./ sqrt(sum(r(p+1:end,:).^2)'*sum(r(p+1:end,:).^2));
Q(:,:,p) = QBar;
r = [r; zeros(1,m)];

% ------------------------------------------------------------------------------------------------
% (5) recursively fill D2, Q, D, and R
% ------------------------------------------------------------------------------------------------

for t = p+1:T+1
    D2(:,:,t) = SigmaBar.*(eye(m)-Lambda_1-Lambda_2) + Lambda_1.*D2(:,:,t-1) + Lambda_2.*diag(r(t-1,:).^2);
    Q(:,:,t)  = QBar.*(1-phi_1-phi_2) + phi_1*Q(:,:,t-1) + phi_2*(rTilde(t-1,:)'*rTilde(t-1,:));
    D(:,:,t) = sqrt(D2(:,:,t));
    if sum(diag(Q(:,:,t) == 0)) > 0
        R(:,:,t) = eye(m);
    else
        R(:,:,t) = Q(:,:,t) ./ (sqrt(diag(Q(:,:,t)))*sqrt(diag(Q(:,:,t)))');
    end
    Sigma(:,:,t) = D(:,:,t)*R(:,:,t)*D(:,:,t);
end

% ------------------------------------------------------------------------------------------------
% (6) compute the log-likelihood
% ------------------------------------------------------------------------------------------------

logLikelihood_t = nan(T, 1);
% warning('off','MATLAB:illConditionedMatrix')
for t = p+1:T
    logLikelihood_t(t) = -m/2*log(pi) - 1/2*log(det(Sigma(:,:,t))) ...
        + gammaln((m + nu)/2) - gammaln(nu/2) - m/2*log(nu-2) ...
        - (m + nu)/2*log(1 + r(t,:)*inv(Sigma(:,:,t))*r(t,:)' / (nu - 2));
end
% warning('on','MATLAB:illConditionedMatrix')

minusLogLikelihood = -nansum(logLikelihood_t(p:end));

% ------------------------------------------------------------------------------------------------
% (7) set the first p observations of Sigma equal to the p+1 th observation
% ------------------------------------------------------------------------------------------------
for t = 1:p
    Sigma(:,:,t) = Sigma(:,:,p+1);
end
