function [parameters, standardErrors, Sigma] = estimate_TDCC(series, nLags, mode, startingValues)

%  ------------------------------------------------------------------------------------------------
%  PURPOSE: estimates the parameters of Pesaran & Pesaran's (2007)
%           t-Dynamic Conditional Correlation (TDCC) model by maximum likelihood
%  ------------------------------------------------------------------------------------------------
%  USAGE: [parameters, standardErrors, Sigma] = estimate_TDCC(series, nLags, mode, startingValues)
%  where: series    = an m x T dimensional series of innovations
%         nLags     = number of lags for computation of the devolatized errors
%         mode      = 'mean-reverting' or 'non-mean-reverting'
%         startingValues = a vector of starting values (optional)
% ------------------------------------------------------------------------------------------------
%  RETURNS: parameters, a vector containing the estimated parameters
%           (1) mean-reverting case:
%                   lambda_1 (univariate GARCH, m-dimensional)
%                   lambda_2 (univariate ARCH, m-dimensional)
%                   phi_1    (GARCH of correlation matrix, scalar)
%                   phi_2    (ARCH of correlation matrix, scalar)
%                   nu     (degrees of freedom of the multivariate t-distribution, scalar)
%           (2) non-mean reverting case:
%                   lambda   (univariate ARCH, m-dimensional)
%                   phi      (ARCH of correlation matrix, scalar)
%                   nu     (degrees of freedom of the multivariate t-distribution, scalar)
%           standardErrors, standard errors of the parameters based on the inverse of the Hessian
%           Sigma, an (m x m x T+1) array containing the conditional covariances
% ------------------------------------------------------------------------------------------------
%  REFERENCE: B. Pesaran and M.H. Pesaran (2007) "Modelling Volatilities and
%       Conditional Correlations in Futures Markets with a Multivariate
%       t Distribution", mimeo.
% ------------------------------------------------------------------------------------------------
%  written by Christoph Schleicher (christoph@iam.ubc.ca)
% ------------------------------------------------------------------------------------------------

if ~exist('startingValues','var'), startingValues = []; end

m = size(series,2);

% ------------------------------------------------------------------------------------------------
% (1) set the starting values and bounds on the parameters
% ------------------------------------------------------------------------------------------------
epsilon = 1e-4;
switch mode
    case 'mean-reverting'
        if m > 1
            lowerBound = [zeros(2*m+2,1)+epsilon; 2.5];
            upperBound = [ones(2*m+2,1)-epsilon; 100];
            A = [eye(m), eye(m), zeros(m, 3);
                zeros(1,2*m), ones(1,2), 0];
            b = ones(m+1,1);
        else
            lowerBound = [zeros(2,1)+epsilon; 2+1e-3];
            upperBound = [ones(2,1)-epsilon; 100];
            A = [1, 1, 0];
            b = 1;
        end
    case 'non-mean-reverting'
        if m > 1
            lowerBound = [zeros(m+1,1)+epsilon; 2+1e-3];
            upperBound = [ones(m+1,1)-epsilon; 100];
        else
            lowerBound = [epsilon; 2+1e-3];
            upperBound = [1-epsilon; 100];
        end
        A = [];
        b = [];
end
if isempty(startingValues)
    switch mode
        case 'mean-reverting'
            if m > 1
                startingValues = [ones(m,1)*0.7; ones(m,1)*0.1; 0.7; 0.1; 10];
            else
                startingValues = [0.7; 0.1; 10];
            end
        case 'non-mean-reverting'
            if m > 1
                startingValues = [ones(m,1)*0.9; 0.9; 10];
            else
                startingValues = [0.9; 10];
            end
    end
end

% ------------------------------------------------------------------------------------------------
% (2) use fmincon to minimize the negative of the log-likelihood
% ------------------------------------------------------------------------------------------------

options = optimset('fmincon');
options = optimset(options, ...
    'Display','iter', ...
    'Diagnostics','off', ...
    'LevenbergMarquardt','off', ...
    'MaxFunEvals', 400 * m, ...
    'MaxIter', 400 * m, ...
    'LargeScale','off', ...
    'MaxSQPIter', 400 * m, ...
    'TolFun',10^(-8), ...
    'TolX',10^(-8));


disp('Estimating TDCC model')
[parameters, fval, exitflag, output, lambda, grad, hessian] = ...
    fmincon('TDCC_likelihood', startingValues, A, b, [], [], lowerBound, upperBound, [], options, series, nLags, mode);
standardErrors = sqrt(diag(inv(hessian)));

% ------------------------------------------------------------------------------------------------
% (3) call the likelihood function again to get the array of conditional covariances, Sigma
% ------------------------------------------------------------------------------------------------

[minusLogLikelihood, Sigma] = TDCC_likelihood(parameters, series, nLags, mode);